@@ -18,7 +18,7 @@ import dateutil.parser
 
 from .. import global_config
 from ..dynamic_loader import iter_classes, iter_modules
-from ..exceptions import PatroniFatalException
+from ..exceptions import PatroniAssertionError, PatroniFatalException
 from ..tags import Tags
 from ..utils import deep_compare, parse_int, uri
 
@@ -198,10 +198,13 @@ class Member(Tags, NamedTuple('Member',
             data = {'conn_url': conn_url, 'api_url': api_url}
         else:
             try:
-                data = json.loads(value)
-                assert isinstance(data, dict)
-            except (AssertionError, TypeError, ValueError):
-                data: Dict[str, Any] = {}
+                json_data = json.loads(value)
+                if isinstance(json_data, dict):
+                    data = cast(Dict[str, Any], json_data)
+                else:
+                    raise PatroniAssertionError('not a dict')
+            except (PatroniAssertionError, TypeError, ValueError):
+                data = {}
         return Member(version, name, session, data)
 
     @property
@@ -480,9 +483,12 @@ class Failover(NamedTuple):
             data: Dict[str, Any] = value
         elif value:
             try:
-                data = json.loads(value)
-                assert isinstance(data, dict)
-            except AssertionError:
+                json_data = json.loads(value)
+                if isinstance(json_data, dict):
+                    data = cast(Dict[str, Any], json_data)
+                else:
+                    raise PatroniAssertionError('not a dict')
+            except PatroniAssertionError:
                 data = {}
             except ValueError:
                 t = [a.strip() for a in value.split(':')]
@@ -547,10 +553,13 @@ class ClusterConfig(NamedTuple):
             False
         """
         try:
-            data = json.loads(value)
-            assert isinstance(data, dict)
-        except (AssertionError, TypeError, ValueError):
-            data: Dict[str, Any] = {}
+            json_data = json.loads(value)
+            if isinstance(json_data, dict):
+                data = cast(Dict[str, Any], json_data)
+            else:
+                raise PatroniAssertionError('not a dict')
+        except (PatroniAssertionError, TypeError, ValueError):
+            data = {}
             modify_version = 0
         return ClusterConfig(version, data, version if modify_version is None else modify_version)
 
@@ -603,11 +612,12 @@ class SyncState(NamedTuple):
         try:
             if value and isinstance(value, str):
                 value = json.loads(value)
-            assert isinstance(value, dict)
+            if not isinstance(value, dict):
+                raise PatroniAssertionError('not a dict')
             leader = value.get('leader')
             quorum = value.get('quorum')
             return SyncState(version, leader, value.get('sync_standby'), int(quorum) if leader and quorum else 0)
-        except (AssertionError, TypeError, ValueError):
+        except (PatroniAssertionError, TypeError, ValueError):
             return SyncState.empty(version)
 
     @staticmethod
@@ -737,10 +747,13 @@ class TimelineHistory(NamedTuple):
             []
         """
         try:
-            lines = json.loads(value)
-            assert isinstance(lines, list)
-        except (AssertionError, TypeError, ValueError):
-            lines: List[_HistoryTuple] = []
+            json_lines = json.loads(value)
+            if isinstance(json_lines, list):
+                lines = cast(List[_HistoryTuple], json_lines)
+            else:
+                raise PatroniAssertionError('not a list')
+        except (PatroniAssertionError, TypeError, ValueError):
+            lines = []
         return TimelineHistory(version, value, lines)
 
 
@@ -1839,10 +1852,6 @@ class AbstractDCS(abc.ABC):
         """
         timestamp = time.time()
 
-        # DCS is a source of truth, therefore we take missing values from there
-        self._last_retain_slots.update({name: timestamp for name in self._last_status['retain_slots']
-                                        if (not slots or name not in slots) and name not in self._last_retain_slots})
-
         if slots:  # if slots is not empty it implies we are running v11+
             members: Set[str] = set()
             found_self = False
@@ -1927,6 +1936,23 @@ class AbstractDCS(abc.ABC):
         :returns: ``True`` if key has been created successfully.
         """
 
+    def acquire_leader_lock(self) -> bool:
+        """Attempt to acquire leader lock.
+
+        .. note::
+            This method wraps :meth:`~AbstractDCS.attempt_to_acquire_leader`: and is
+            used to reset retention time of physical replication slots that representing
+            members of the cluster when current node is to be promoted to the leader.
+
+        :returns: ``True`` if the leader key has been created successfully.
+        """
+        ret = self.attempt_to_acquire_leader()
+        if ret:
+            timestamp = time.time()
+            # every time we promote we need to reset retention time for slots recorded in the /status key
+            self._last_retain_slots = {name: timestamp for name in self._last_status['retain_slots']}
+        return ret
+
     @abc.abstractmethod
     def set_failover_value(self, value: str, version: Optional[Any] = None) -> bool:
         """Create or update ``/failover`` key.
