@@ -83,6 +83,8 @@ class Postgresql(object):
         self._connection = self.connection_pool.get('heartbeat')
         self.mpp_handler = mpp.get_handler_impl(self)
         self._bin_dir = config.get('bin_dir') or ''
+        self._role_lock = Lock()
+        self.set_role('uninitialized')
         self.config = ConfigHandler(self, config)
         self.config.check_directories()
 
@@ -107,7 +109,6 @@ class Postgresql(object):
         self._is_leader_retry = Retry(max_tries=1, deadline=config['retry_timeout'] / 2.0, max_delay=1,
                                       retry_exceptions=PostgresConnectionException)
 
-        self._role_lock = Lock()
         self.set_role(self.get_postgres_role_from_data_directory())
         self._state_entry_timestamp = 0
 
@@ -1328,7 +1329,7 @@ class Postgresql(object):
                     os.unlink(source)
                     os.symlink(new_name, source)
 
-                new_name = '{0}.{1}'.format(self._data_dir, postfix)
+                new_name = '{0}.{1}'.format(self._data_dir.rstrip(os.sep), postfix)
                 logger.info('renaming data directory to %s', new_name)
                 if os.path.exists(new_name):
                     shutil.rmtree(new_name)
