@@ -1102,8 +1102,11 @@ class ConfigHandler(object):
         listen_addresses = self._server_parameters['listen_addresses'].split(',')
 
         for la in listen_addresses:
-            if la.strip().lower() in ('*', '0.0.0.0', '127.0.0.1', 'localhost'):  # we are listening on '*' or localhost
+            if la.strip().lower() in ('*', 'localhost'):  # we are listening on '*' or localhost
                 return 'localhost'  # connection via localhost is preferred
+            if la.strip() in ('0.0.0.0', '127.0.0.1'):  # Postgres listens only on IPv4
+                # localhost, but don't allow Windows to resolve to IPv6
+                return '127.0.0.1' if os.name == 'nt' else 'localhost'
         return listen_addresses[0].strip()  # can't use localhost, take first address from listen_addresses
 
     def resolve_connection_addresses(self) -> None:
