@@ -1,7 +1,9 @@
-import psutil
 import unittest
 
-from mock import Mock, patch
+from unittest.mock import Mock, patch
+
+import psutil
+
 from patroni.postgresql.callback_executor import CallbackExecutor
 
 
