@@ -228,14 +228,13 @@ class TestValidator(unittest.TestCase):
         c["kubernetes"]["pod_ip"] = "::1"
         c["consul"]["host"] = "127.0.0.1:50000"
         c["etcd"]["host"] = "127.0.0.1:237"
-        c["postgresql"]["listen"] = "127.0.0.1:5432"
         with patch('patroni.validator.open', mock_open(read_data='9')):
             errors = schema(c)
         output = "\n".join(errors)
-        self.assertEqual(['consul.host', 'etcd.host', 'postgresql.bin_dir', 'postgresql.data_dir', 'postgresql.listen',
+        self.assertEqual(['consul.host', 'etcd.host', 'postgresql.bin_dir', 'postgresql.data_dir',
                           'raft.bind_addr', 'raft.self_addr', 'restapi.connect_address'], parse_output(output))
 
-    def test_bin_dir_is_empty_string_excutables_in_path(self, mock_out, mock_err):
+    def test_bin_dir_is_empty_string_executables_in_path(self, mock_out, mock_err):
         binaries.extend(required_binaries)
         c = copy.deepcopy(config)
         c["postgresql"]["bin_dir"] = ""
@@ -259,7 +258,7 @@ class TestValidator(unittest.TestCase):
         self.assertEqual(['raft.bind_addr', 'raft.self_addr'], parse_output(output))
 
     @patch('subprocess.check_output', Mock(return_value=b"postgres (PostgreSQL) 12.1"))
-    def test_pg_version_missmatch(self, mock_out, mock_err):
+    def test_pg_version_mismatch(self, mock_out, mock_err):
         directories.append(config["postgresql"]["data_dir"])
         directories.append(config["postgresql"]["bin_dir"])
         directories.append(os.path.join(config["postgresql"]["data_dir"], "pg_wal"))
